
package vpn;

import java.io.BufferedReader;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.ObjectOutputStream;
import java.lang.reflect.Field;
import java.security.Key;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.SecureRandom;
import java.security.Security;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;
import javax.crypto.Cipher;
import org.bouncycastle.jce.provider.BouncyCastleProvider;

/**
 *
 * @author cchrysler
 */
public class KeyGenerator implements Runnable {

    String confFileLocation = "";
    String pubKeyName = "";
    String privKeyName = "";
    Map<String, String> confMap = new HashMap();
    List<String> unsetFields = new ArrayList();

    /** Creates a new instance of KeyGenerator */
    public KeyGenerator(String _confFileLocation,
            String _privKeyName,
            String _pubKeyName) {
        confFileLocation = _confFileLocation;
        privKeyName = _privKeyName;
        pubKeyName = _pubKeyName;
    }

    private void printBytesAsHex(byte[] buffer) {
        for (int i = 0; i < buffer.length; i++) {
            System.out.print(toHexString(buffer[i], 2));
        }
        System.out.println();
    }

    private void printBytesAsString(byte[] buffer) {
        for (int i = 0; i < buffer.length; i++) {
            System.out.print((char) buffer[i]);
        }
        System.out.println();
    }

    private String toHexString(long n, int length) {
        String result = "";
        for (int i = 0; i < length; i++) {
            result = Integer.toHexString(((int) n) & 0xF) + result;
            n >>>= 4;
        }
        return result;
    }

    public void parseConf(String fileLocation) throws Exception {

        BufferedReader bfr = new BufferedReader(new FileReader(fileLocation));
        while (bfr.ready()) {
            StringBuffer sb = new StringBuffer();
            String line = bfr.readLine();
            for (int i = 0; i < line.length(); i++) {
                if (line.charAt(i) != '#') {
                    sb.append(line.charAt(i));
                } else {
                    break;
                }
            }
            if (Pattern.matches("^(.)+(=)(.)+$", sb.toString())) {
                String[] sa = sb.toString().split("=");
                confMap.put(sa[0].trim(), sa[1].trim());
            }
        }
        bfr.close();
    }

    public void run() {
        try {

            this.parseConf(confFileLocation);
            ConfBean cb = ConfBean.getInstance();
            Class c = cb.getClass();
            Field[] confFields = c.getFields();

            for (Field f : confFields) {
                f.set(cb, confMap.get(f.getName()));
            }

            for (Field f : confFields) {
                if (confMap.get(f.getName()) == null) {
                    unsetFields.add(f.getName());
                } else if (confMap.get(f.getName()).equals("")) {
                    unsetFields.add(f.getName());
                }
            }

            Security.addProvider(new BouncyCastleProvider());

            String test = "You suck!_" + System.currentTimeMillis();

            KeyPairGenerator generator =
                    KeyPairGenerator.getInstance("RSA");
            generator.initialize(1024);

            byte[] input = test.getBytes();

            Cipher cipher = Cipher.getInstance("RSA/ECB/PKCS1Padding", "BC");//("RSA");
            SecureRandom random = new SecureRandom();
            byte bytes[] = new byte[25];
            random.nextBytes(bytes);
            generator.initialize(1024, random);

            KeyPair pair = generator.generateKeyPair();
            Key pubKey = pair.getPublic();
            Key privKey = pair.getPrivate();

            System.out.print("pre-encryption : ");
            printBytesAsString(input);

            // encryption step

            cipher.init(Cipher.ENCRYPT_MODE, pubKey);//, random);

            byte[] cipherText = cipher.doFinal(input);


            System.out.print("cipher: ");
            printBytesAsHex(cipherText);

            // decryption step
            cipher.init(Cipher.DECRYPT_MODE, privKey);

            byte[] plainText = cipher.doFinal(cipherText);

            System.out.print("post-encryption : ");
            printBytesAsString(plainText);

            ObjectOutputStream out = new ObjectOutputStream(
                    new FileOutputStream(cb.getPubKeyDir() + "/" + pubKeyName));
            out.writeObject(pubKey);
            out.flush();

            out = new ObjectOutputStream(
                    new FileOutputStream(cb.getPrivKeyDir() + "/" + privKeyName));
            out.writeObject(privKey);
            out.flush();
            out.close();


        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public static void main(String[] args) {
        if (args.length == 3) {
            Thread t = new Thread(new KeyGenerator(args[0], args[1], args[2]));
            t.start();
        }
        else {
            System.out.println("expecting 3 command line parameters: <Conf file location> , <Priv keyname> , <Pub keyname>");
        }
    }
}
