
package vpn;

import java.io.BufferedReader;
import java.io.FileReader;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;
import jpcap.NetworkInterface;
import jpcap.JpcapCaptor;
import jpcap.PacketReceiver;
import jpcap.packet.Packet;

/**
 *
 * @author cchrysler
 */
public class SimpleSPA implements Runnable, PacketReceiver {

    String confFileLocation = "";
    Map<String, String> confMap = new HashMap();
    List<String> unsetFields = new ArrayList();

    /** Creates a new instance of JpcapTest */
    public SimpleSPA() {
    }

    public SimpleSPA(String fileLocation) {
        confFileLocation = fileLocation;
    }

    public void receivePacket(Packet packet) {

        //System.out.println("srcIp = " + parseIP(packet));
        //printBytesAsHex(packet.data);
        String[] params = {parseIP(packet), printBytesAsHex(packet.data),
            confFileLocation
        };
        PacketReader.main(params);

    }

    public String parseIP(Packet _packet) {

        String ret = "";
        try {
            String[] firstSplit = _packet.toString().split("(->)");
            String[] secondSplit = firstSplit[0].split("/");
            ret = secondSplit[1];
        } catch (Exception e) {
            System.out.println(e.toString());
        }
        return ret;

    }
    
    private String printBytesAsHex(byte[] buffer) {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < buffer.length; i++) {
            sb.append(toHexString(buffer[i], 2));
            //System.out.print(toHexString(buffer[i], 2));

        }

        //System.out.println();
        return sb.toString();

    }

    private String toHexString(long n, int length) {

        String result = "";
        for (int i = 0; i < length; i++) {
            result = Integer.toHexString(((int) n) & 0xF) + result;
            n >>>= 4;
        }

        return result;

    }

    public void parseConf(String fileLocation) throws Exception {

        BufferedReader bfr = new BufferedReader(new FileReader(fileLocation));
        while (bfr.ready()) {
            StringBuffer sb = new StringBuffer();
            String line = bfr.readLine();
            for (int i = 0; i < line.length(); i++) {
                if (line.charAt(i) != '#') {
                    sb.append(line.charAt(i));
                } else {
                    break;
                }
            }
            if (Pattern.matches("^(.)+(=)(.)+$", sb.toString())) {
                String[] sa = sb.toString().split("=");
                confMap.put(sa[0].trim(), sa[1].trim());
            }
        }
        bfr.close();
    }

    public void run() {
        try {

            this.parseConf(confFileLocation);
            ConfBean cb = ConfBean.getInstance();
            Class c = cb.getClass();
            Field[] confFields = c.getFields();

            for (Field f : confFields) {
                f.set(cb, confMap.get(f.getName()));
            }

            for (Field f : confFields) {
                if (confMap.get(f.getName()) == null) {
                    unsetFields.add(f.getName());
                } else if (confMap.get(f.getName()).equals("")) {
                    unsetFields.add(f.getName());
                }
            }

            SimpleLogger logger = SimpleLogger.getInstance();

            if (!unsetFields.isEmpty()) {
                for (String s : unsetFields) {
                    logger.logMessage("Error in conf file, " + s + " not set to a value");
                    System.out.println(s + " was not set in the conf file....quitting");
                }
                throw new Exception();
            }
            

            NetworkInterface device = new NetworkInterface(/*"eth0"*/cb.getInterfaceOfListener(), "", false, "", "", //parameterized from conf
                    null, null);
            JpcapCaptor captor = JpcapCaptor.openDevice(device, 65535, false, 20);//parameterized from conf            
            captor.setFilter("src or dst port " + cb.getPortOfListener()/*"22222"*/, true);//parameterized from conf
            logger.logMessage("Jpcap running!");
            captor.loopPacket(-1, new SimpleSPA());
            captor.close();
            
        } catch (Exception e) {
            System.out.println(e.toString());
        }

    }

    public static void main(String[] args) {
        Thread t = new Thread(new SimpleSPA(args[0]));
        t.start();
    }
}
